require "active_support/all"

module JSON::API
  def json_params
    begin
      input = request.body.gets
      @json_params ||= input ? JSON.parse(input) : {}
      request.params.merge(@json_params)
      # request.params = @json_params
    rescue
      halt 400, { message: 'Invalid JSON params' }.to_json
    end
  end

  DATE_GENERATOR = {
    "yesterday" => ->() { DateTime.now - 24.hours },
    "now"       => ->() { DateTime.now },
    "today"     => ->() { DateTime.now },
    nil         => ->() { DateTime.now },
  }
  # @param [String] date: either a date iso formatted or a word to be sent to DateTime
  # @param [Float] offset: an amount of hours to remove from the date, useful for setting the start of the day hours after midnight
  def date_input_convertor(date = "now", offset = 0.0)
    base_time =
      if DATE_GENERATOR.key?(date)
        DATE_GENERATOR[date].call()
      else
        begin
          DateTime.parse date
        rescue => _
          DateTime.now
        end
      end
    (base_time - offset.hours).to_date
  end

  # params:
  #   - mimes: example: %w(application/json text/json)
  def accept?(*mimes)
    @request_accept_str ||= request.accept.map(&:to_str)
    @request_accept_str.intersection(mimes).size > 0
  end

  def accept_json?
    accept? 'application/json'
  end

  def accept_html?
    accept? 'text/html'
  end

  def slim_partial(partial, locals: {})
    Slim::Template.new(
      "src/views/partials/#{partial}.slim",
      # layout: false,
    ).render(self, locals)
  end

  # Example
  # render do
  #   json "{ ... }"
  #   html "< ... >"
  # end
  def given_content_type(&block)
    raise "Invalid DSL call to JSON::API.render without block" unless block_given?
    Render.new(self, &block).complete_rendering
  end

  class Render
    delegate :slim,
             :slim_partial,
    #          :api_error,
    #          :api_response,
    #          :render,
             to: :@context
    attr_reader :context

    def initialize(context, &block)
      @render = nil
      @context = context
      instance_eval(&block) if block_given?
    end

    def render_something_if_accept(acceptance, output, &block)
      if block_given?
        context.instance_eval(&block)
      else
        @render = output
      end if @context.send("accept_#{acceptance}?")
    end

    def json(output = nil, &block)
      render_something_if_accept :json, output, &block
    end

    def html(output = nil, &block)
      render_something_if_accept :html, output, &block
    end

    def default(output = nil, &block)
      if block_given?
        @render_default_block = block
      else
        @render_default = output
      end
    end

    def complete_rendering
      @render ||
        (@render_default_block.is_a?(Proc) && @context.instance_eval(&@render_default_block)) ||
        @render_default
    end
  end
end
